@@ -7,7 +7,9 @@ use warnings;
 =encoding utf8
 
 =cut
+
 use Moo;
+use Types::Standard qw(Str);
 
 with 'Net::Etcd::Role::Actions';
 use namespace::clean;
@@ -18,7 +20,7 @@ Net::Etcd::Maintenance
 
 =cut
 
-our $VERSION = '0.014';
+our $VERSION = '0.015';
 
 =head1 DESCRIPTION
 
@@ -26,6 +28,17 @@ Provides support for maintenance related actions.
 
 =cut
 
+=head1 ACCESSORS
+
+=head2 endpoint
+
+=cut
+
+has endpoint => (
+    is      => 'rwp',
+    isa     => Str,
+);
+
 =head1 PUBLIC METHODS
 
 =head2 snapshot
